@@ -17,10 +17,12 @@ setMethod("names<-", signature(x="SpatRaster"),
 		if (length(value) != nlyr(x)) {
 			stop("incorrect number of names")
 		}
-		if (! x@ptr$setNames(value)) {
+		if (! x@ptr$setNames(value, FALSE)) {
 			stop("cannot set these names")
 		}
+		
 		if (any(names(x) != value)) {
+			# should only be possible with $setNames(value, TRUE)
 			warning("some names were changed to make them valid and/or unique")
 		}
 		return(x)
