@@ -56,8 +56,8 @@
 }
 
 .get.leg.extent <- function(x) {
-	usr <- graphics::par()$usr
-	dxy <- graphics::par()$cxy * graphics::par("cex")	
+	usr <- graphics::par("usr")
+	dxy <- graphics::par("cxy") * graphics::par("cex")	
 	loc <- x$leg$loc
 	p <- NULL
 	if (is.character(loc)) {
@@ -133,6 +133,10 @@
 		zz <- pretty(zlim, n =(x$levels+1))	
 		zz <- zz[zz >= zlim[1] & zz <= zlim[2]]
 	}
+	zztxt <- x$leg$labels
+	if (is.null(zztxt)) {
+		zztxt <- formatC(zz, digits=x$leg$digits, format = "f")
+	}
 	e <- x$leg$ext
 	if (x$leg$loc %in% c("left", "right")) {
 		Y <- seq(e$ymin, e$ymax, length.out=nc+1)
@@ -140,10 +144,10 @@
 		ypos <- e$ymin + (zz - zlim[1])/(zlim[2] - zlim[1]) * e$dy
 		if (x$leg$loc == "right") {
 			graphics::segments(e$xmin, ypos, e$xmax+e$dx*0.25, ypos, xpd=NA)
-			text(e$xmax, ypos, formatC(zz, digits=x$leg$digits, format = "f"), pos=4, xpd=NA, cex=cex, ...)
+			text(e$xmax, ypos, zztxt, pos=4, xpd=NA, cex=cex, ...)
 		} else {
 			graphics::segments(e$xmin-e$dx*0.25, ypos, e$xmax, ypos, xpd=NA)
-			text(e$xmin, ypos, formatC(zz, digits=x$leg$digits, format = "f"), pos=2, xpd=NA, cex=cex, ...)
+			text(e$xmin, ypos, zztxt, pos=2, xpd=NA, cex=cex, ...)
 		}
 	} else {
 		X <- seq(e$xmin, e$xmax, length.out=nc+1)
@@ -151,16 +155,15 @@
 		xpos <- e$xmin + (zz - zlim[1])/(zlim[2] - zlim[1]) * e$dx
 		if (x$leg$loc == "bottom") {
 			graphics::segments(xpos, e$ymin-e$dy*0.25, xpos, e$ymax, xpd=NA)
-			text(xpos, e$ymin, formatC(zz, digits=x$leg$digits, format = "f"), pos=1, xpd=NA, cex=cex)
+			text(xpos, e$ymin, zztxt, pos=1, xpd=NA, cex=cex)
 		} else {
 			graphics::segments(xpos, e$ymin, xpos, e$ymax+e$dy*0.25, xpd=NA)
-			text(xpos, e$ymax+e$dy*0.25, formatC(zz, digits=x$leg$digits, format = "f"), pos=3, xpd=NA, cex=cex)
+			text(xpos, e$ymax+e$dy*0.25, zztxt, pos=3, xpd=NA, cex=cex)
 		}
 	}	
 	graphics::rect(e$xmin, e$ymin, e$xmax, e$ymax, border ="black", xpd=NA)
 	
 	x <- .leg.main(x)
-	
 	x
 }	
 
