@@ -105,6 +105,8 @@ setMethod("predict", signature(object="SpatRaster"),
 		nl <- 1
 		nc <- ncol(object)
 		tomat <- FALSE
+		if (!readStart(object)) { stop(object@ptr$messages$getError()) }
+		on.exit(readStop(object))
 		d <- readValues(object, round(0.5*nrow(object)), 1, 1, min(nc,500), TRUE, TRUE)
 	
 		r <- .runModel(model, fun, d, nl, const, na.rm, index, ...)
@@ -113,15 +115,13 @@ setMethod("predict", signature(object="SpatRaster"),
 		cn <- colnames(r)
 		if (length(cn) == nl) names(out) <- make.names(cn, TRUE)
 		
-		readStart(object)
 		b <- writeStart(out, filename, overwrite, wopt)
 		for (i in 1:b$n) {
 			d <- readValues(object, b$row[i], b$nrows[i], 1, nc, TRUE, TRUE)
 			r <- .runModel(model, fun, d, nl, const, na.rm, index, ...)
 			writeValues(out, r, b$row[i], b$nrows[i])
 		}
-		readStop(object)
-		out <- writeStop(out)
+		writeStop(out)
 		return(out)
 	}
 )
