@@ -239,7 +239,7 @@ SpatMessages transform_coordinates(std::vector<double> &x, std::vector<double> &
 	poCT = OGRCreateCoordinateTransformation(&source, &target);
 
 	if( poCT == NULL )	{
-		m.setError( "Transformation failed" );
+		m.setError( "Cannot do this transformation" );
 		return (m);
 	}
 
@@ -258,7 +258,73 @@ SpatMessages transform_coordinates(std::vector<double> &x, std::vector<double> &
 }
 
 
+SpatVector SpatVector::project(std::string crs) {
 
+	SpatVector s;
+
+    #ifndef useGDAL
+		s.setError("GDAL is not available");
+		return(s);
+	#else
+
+	OGRSpatialReference source, target;
+	std::string vsrs = getSRS("wkt");
+	const char *pszDefFrom = vsrs.c_str();
+	OGRErr erro = source.SetFromUserInput(pszDefFrom);
+	if (erro != OGRERR_NONE) {
+		s.setError("input crs is not valid");
+		return s;
+	}
+
+	const char *pszDefTo = crs.c_str();
+	erro = target.SetFromUserInput(pszDefTo);
+	if (erro != OGRERR_NONE) {
+		s.setError("output crs is not valid");
+		return s;
+	}
+
+	CPLSetConfigOption("OGR_CT_FORCE_TRADITIONAL_GIS_ORDER", "YES");
+	OGRCoordinateTransformation *poCT;
+	poCT = OGRCreateCoordinateTransformation(&source, &target);
+
+	if( poCT == NULL )	{
+		s.setError( "Cannot do this transformation" );
+		return(s);
+	}
+
+	s.setSRS(crs);
+	s.df = df;
+	std::vector<unsigned> keeprows;
+	
+	for (size_t i=0; i < size(); i++) {
+		SpatGeom g = getGeom(i);
+		SpatGeom gg;
+		gg.gtype = g.gtype;
+		for (size_t j=0; j < g.size(); j++) {
+			SpatPart p = g.getPart(j);
+			if (poCT->Transform(p.x.size(), &p.x[0], &p.y[0]) ) {
+				SpatPart pp(p.x, p.y);
+				if (p.hasHoles()) {
+					for (size_t k=0; k < p.nHoles(); k++) {
+						SpatHole h = p.getHole(k);
+						poCT->Transform(h.x.size(), &h.x[0], &h.y[0]);
+						pp.addHole(h.x, h.y);
+					}
+				}
+				gg.addPart(pp);
+				keeprows.push_back(j);
+			}
+		}
+		s.addGeom(gg);
+		s.df.subset_rows(keeprows);		
+	}
+
+	#endif
+	return s;
+}
+
+
+/*
 SpatVector SpatVector::project(std::string crs) {
 
 	SpatVector s;
@@ -284,12 +350,22 @@ SpatVector SpatVector::project(std::string crs) {
 	if (!s.msg.has_error) {
 		unsigned n = d.iv[0].size();
 		std::vector<unsigned> a, b, c;
+		std::vector<double> ptx, pty;
+		a.reserve(n);
+		b.reserve(n);
+		c.reserve(n);
+		ptx.reserve(n);
+		pty.reserve(n);
 		for (size_t i=0; i<n; i++) {
-			a.push_back(d.iv[0][i]);
-			b.push_back(d.iv[1][i]);
-			c.push_back(d.iv[2][i]);
+			if (!std::isnan(x[i])) {
+				a.push_back(d.iv[0][i]);
+				b.push_back(d.iv[1][i]);
+				c.push_back(d.iv[2][i]);
+				ptx.push_back(x[i]);
+				pty.push_back(y[i]);
+			}
 		}
-		s.setGeometry(type(), a, b, x, y, c);
+		s.setGeometry(type(), a, b, ptx, pty, c);
 		//std::vector<std::string> refs = srefs_from_string(crs);
 		std::string msg;
 		s.setSRS(crs);
@@ -300,5 +376,7 @@ SpatVector SpatVector::project(std::string crs) {
 	return s;
 }
 
+*/
+
 #endif
 
