@@ -44,22 +44,24 @@ double availableRAM() {
 		sysinfo (&memInfo);
 		ram = memInfo.freeram;
 	#elif __APPLE__
+
 		vm_size_t page_size;
 		mach_port_t mach_port;
 		mach_msg_type_number_t count;
 		vm_statistics64_data_t vm_stats;
+
 		mach_port = mach_host_self();
 		count = sizeof(vm_stats) / sizeof(natural_t);
 		if (KERN_SUCCESS == host_page_size(mach_port, &page_size) &&
-					KERN_SUCCESS == host_statistics64(mach_port, HOST_VM_INFO,
+			KERN_SUCCESS == host_statistics64(mach_port, HOST_VM_INFO,
 											(host_info64_t)&vm_stats, &count)) {
-			ram = (int64_t)vm_stats.free_count * (int64_t)page_size;
-			
-			ram = ram * 5; // purely empirical. The estimate seemed much too low. Needs work.
+			long long free_memory = ((int64_t)vm_stats.free_count +
+                               (int64_t)vm_stats.inactive_count) * (int64_t)page_size;
+			ram = free_memory;
+		//https://stackoverflow.com/questions/63166/how-to-determine-cpu-and-memory-consumption-from-inside-a-process
 		}
 		
 	#endif
 	return ram / 8;  // 8 bytes for each double
 }
 
-
